@@ -25,6 +25,7 @@
 #include <common.h>
 #include <prom_no.h>
 #include <ambapp_ids.h>
+#include <config.h>
 
 /* AMBA Nodes */
 extern struct node *pnodes_amba;
@@ -56,13 +57,8 @@ extern void ambapp_freq_init(int ahb_bus_idx, unsigned long freq_hz);
 #define CONFIG_AMBA_IO_AREA AMBA_DEFAULT_IOAREA
 #endif
 
-#ifndef CONFIG_AMBA_AHB_MAX
-#define AMBA_AHB_MAX 6
-#endif
-
-#ifndef CONFIG_AMBA_APB_MAX
-#define AMBA_APB_MAX 6
-#endif
+#define AMBA_AHB_MAX CONFIG_AMBA_AHB_MAX
+#define AMBA_APB_MAX CONFIG_AMBA_APB_MAX
 
 /* Structures used to access Plug&Play information directly */
 struct ambapp_pnp_axb {
