@@ -24,7 +24,7 @@
  * Any changes/bug-fixes should be made in the script, then rerun the script
  * to create a new file.
  *
- * Created from GRLIB revision 4297
+ * Created from GRLIB revision 4299
  */
 
 #ifndef _ambapp_ids_h_
@@ -72,6 +72,7 @@
 #define VENDOR_AAC           0xaa
 #define VENDOR_UC_BERKELEY   0xab
 #define VENDOR_ACTEL         0xac
+#define VENDOR_BG            0xad
 #define VENDOR_APPLECORE     0xae
 #define VENDOR_C3E           0xc3
 #define VENDOR_CBKPAN        0xc8
@@ -79,6 +80,9 @@
 #define VENDOR_CETON         0xcb
 #define VENDOR_SSTL          0xd6
 #define VENDOR_EMBEDDIT      0xea
+#define VENDOR_BRAINCHIP     0xf1
+#define VENDOR_MAGICS        0xf2
+#define VENDOR_LEAT          0xf3
 #define VENDOR_NASA_GSFC     0xfc
 #define VENDOR_AZST          0xfe
 
@@ -307,6 +311,10 @@
 #define GAISLER_GRLSEDC      0x0de
 #define GAISLER_MDIO_CTRL    0x0df
 #define GAISLER_OTP_WRPR     0x0e0
+#define GAISLER_LATTICE_DDR3 0x0e1
+#define GAISLER_GR1553B_LRAM 0x0e2
+#define GAISLER_GRETH2       0x0e5
+#define GAISLER_GR765THS     0x0e6
 
 
 /* ESA
@@ -426,6 +434,12 @@
 #define ORBITA_USBDEV        0x2bc
 
 
+/* SYNOPSYS
+ * Synopsys Inc. device ID defines
+ */
+#define SYNOPSYS_PCIE_CTRL   0x001
+
+
 /* NASA
  * NASA device ID defines
  */
@@ -438,6 +452,7 @@
 #define GAISLER_AHBMST_EM    0x040
 #define GAISLER_TEST_1X2     0x078
 #define ST_MIPHY             0x001
+#define ST_AIB               0x002
 
 
 /* TTTECH
@@ -513,6 +528,15 @@
 #define ACTEL_PFFDDR4        0x00c
 
 
+/* BG
+ * Beyond Gravity GmbH device ID defines
+ */
+#define BG_RIXCORE           0x001
+#define BG_DIAGIF            0x002
+#define BG_WDOG              0x003
+#define BG_SPI               0x004
+
+
 /* APPLECORE
  * AppleCore device ID defines
  */
@@ -544,6 +568,12 @@
 #define SSTL_ZERO_MEM        0x003
 
 
+/* LEAT
+ * LEAT device ID defines
+ */
+#define LEAT_SPLEAT          0x001
+
+
 /* SYSTEMS
  * System ID defines
  */
@@ -589,6 +619,7 @@
 #define XILINX_KC705         0xa705
 #define XILINX_VC707         0xa707
 #define ESA_SSDP             0xada2
+#define NOELV_XILINX_VCU118  0x0402
 
 
 #endif /* _ambapp_ids_h_ */
