@@ -24,6 +24,11 @@
 
 #include <stdint.h>
 
+/* Bit 0-2 mask for reading # timers from the conf register */
+#define GPTIMER_CONF_TIMERS_MASK 0x7
+/* Bit 8 indicates if the timer unit generates separate interrupts */
+#define GPTIMER_CONF_SEP_IRQ (1 << 8)
+
 struct gptimer_timer {
 	uint32_t val;
 	uint32_t rld;
