@@ -30,6 +30,9 @@
 #ifdef CONFIG_WATCHDOG
 #include <prom_watchdog.h>
 #endif
+#ifdef CONFIG_HRTIMERS
+#include <cpucounter.h>
+#endif
 
 static void leon_reboot(char *bcommand);
 static void leon_halt(void);
@@ -477,15 +480,6 @@ static void opprom_setup_banner_name(void)
 
 #ifdef HASXML
 
-static void node_add_child(struct node *node, struct node *child)
-{
-	struct node **np = &node->child;
-
-	while (*np)
-		np = &((*np)->sibling);
-	*np = child;
-}
-
 static void xml_add_root_subnodes(void)
 {
 	struct node *root = node_root;
@@ -512,6 +506,122 @@ static void xml_add_root_subnodes(void) {
 
 #endif /* HASXML */
 
+#ifdef CONFIG_HRTIMERS
+
+extern int hrtimers_res_ok(void);
+
+/* Common clock related: */
+unsigned long clock_cells = 0;
+
+/* System clock related: */
+unsigned long node_sysclk = -1;
+unsigned long leon_freq_hz = 0;
+
+struct propa_ptr props_sysclk[] = {
+       PROPA_PTR("name", "sysclk", 7),
+       PROPA_PTR("compatible", "fixed-clock", 12),
+       PROPA_PTR("clock-frequency", &leon_freq_hz, 4),
+       PROPA_PTR("#clock-cells", &clock_cells, 4),
+       PROPA_PTR_END("clock-output-names", "sysclk", 7),
+};
+
+/* GPTIMER0 clock properties: */
+unsigned long node_gptimer0_clk = -1;
+unsigned long gptimer0_clk_div = 1;
+unsigned long gptimer0_clk_mult = 1;
+
+struct propa_ptr props_gptimer0_clk[] = {
+       PROPA_PTR("name", "gptimer0_clk", 13),
+       PROPA_PTR("compatible", "fixed-factor-clock", 19),
+       PROPA_PTR("clock-mult", &gptimer0_clk_mult, 4),
+       PROPA_PTR("clock-div", &gptimer0_clk_div, 4),
+       PROPA_PTR("#clock-cells", &clock_cells, 4),
+       PROPA_PTR_END("clocks", &node_sysclk, 4),
+};
+
+/* LEON up-counter properties: */
+unsigned long upcounter_width = LEON_UP_COUNTER_MIN_WIDTH;
+
+struct propa_ptr props_leon_cs[] = {
+       PROPA_PTR("name", "clksrc_leon", 12),
+       PROPA_PTR("gaisler,upcounter-width", &upcounter_width, 4),
+       PROPA_PTR("clocks", &node_sysclk, 4),
+       PROPA_PTR_END("compatible", "gaisler,leon-cs", 16),
+};
+
+struct node nodes_lvl2_hrtimer[] = {
+	{NULL, &nodes_lvl2_hrtimer[1], (PROP)props_sysclk},
+	{NULL, &nodes_lvl2_hrtimer[2], (PROP)props_gptimer0_clk},
+	{NULL, NULL, (PROP)props_leon_cs},
+};
+
+#define NODE_SYSCLK_IDX 0
+#define NODE_GPTIMER0_CLK_IDX 1
+
+static inline int leon_up_counter_node_init(void)
+{
+	if (!LEON_UP_COUNTER_AVAIL())
+		return -1;
+
+	/* Unfortunately its not possible to probe the width of the upcounter */
+	if (amba_systemid_is_gr740())
+		upcounter_width = 56;
+
+	return 0;
+}
+
+static inline void clocks_sysclk_node_init(void)
+{
+	leon_freq_hz = int_mul(cpu_freq_khz, 1000);
+	node_sysclk = (unsigned long)&nodes_lvl2_hrtimer[0];
+}
+
+static inline int clocks_gptimer0_clk_node_init(void)
+{
+	unsigned int scaler;
+
+	if (!gptimer_inited)
+		return -1;
+
+	scaler = BYPASS_LOAD_PA(&gptimer->scaler_reload);
+
+	gptimer0_clk_div = scaler + 1;
+
+	node_gptimer0_clk = (unsigned long)&nodes_lvl2_hrtimer[1];
+
+	return 0;
+}
+
+void hrtimers_setup_clk_nodes(void)
+{
+	int ret;
+
+	/* Skip clocks if the timer allocation failed */
+	if (!hrtimers_res_ok())
+		return;
+
+	/* Setup the system clock node */
+	clocks_sysclk_node_init();
+
+	/* Setup the LEON up-counter clocksource */
+	ret = leon_up_counter_node_init();
+	if (ret) {
+		/* up-counter not available, unlink it. */
+		nodes_lvl2_hrtimer[NODE_GPTIMER0_CLK_IDX].sibling = NULL;
+	}
+
+	/* Setup the GPTIMER0 clock */
+	ret = clocks_gptimer0_clk_node_init();
+	if (ret) {
+		/* No gptimer0 available, update sysclock's sibling. */
+		nodes_lvl2_hrtimer[NODE_SYSCLK_IDX].sibling =
+			nodes_lvl2_hrtimer[NODE_GPTIMER0_CLK_IDX].sibling;
+	}
+
+	/* Add the nodes to root */
+	node_add_child(node_root, nodes_lvl2_hrtimer);
+}
+#endif
 /* PROM Initialization routine, called from STARTUP when previous stages has
  * been successfully initialized
  */
@@ -525,15 +635,7 @@ void opprom_init(void)
 	avail.num_bytes = common.end_of_mem_linux_pa - CONFIG_RAM_START;
 	totphys.num_bytes = avail.num_bytes;
 
-	/* Get Number of CPUs */
-#ifndef CONFIG_SMP
-	ncpu = 1;
-#else
-	ncpu = ((BYPASS_LOAD_PA(&irqmp->mpstatus) >> IRQMP_STATUS_CPUNR) & 0xf)
-		+ 1;
-	if (ncpu > CONFIG_MAX_CPUS)
-		ncpu = CONFIG_MAX_CPUS;
-#endif
+        ncpu = irqmp_numcpus();
 
 #if (CONFIG_WAKE_CPU_MASK != 0)
 	/* Wake other (disabled) CPUs if user has requested it */
@@ -549,6 +651,9 @@ void opprom_init(void)
 
 	xml_add_root_subnodes();
 
+#ifdef CONFIG_HRTIMERS
+	hrtimers_setup_clk_nodes();
+#endif
 	opprom_setup_banner_name();
 
 	/* Calculate IDPROM Checksum */
