@@ -56,6 +56,16 @@ char *prop_str_clock_indices = "clock-indices";
 char *prop_str_clocks = "clocks";
 char *prop_str_clock_names = "clock-names";
 
+#ifdef CONFIG_HRTIMERS
+char *prop_str_cs_tmr_idx = "gaisler,cs-timer-idx";
+char *prop_str_tmr_offset = "gaisler,timer-offset";
+char *prop_str_tmr_len = "gaisler,timer-len";
+char *prop_str_cpu_offset = "gaisler,cpu-offset";
+char *prop_str_compatible_timer = "gaisler,gptimer-timer";
+char *prop_str_compatible_timer_cs = "gaisler,gptimer-timer-cs";
+char *prop_str_compatible_timer_ce = "gaisler,gptimer-timer-ce";
+#endif
+
 #ifdef CONFIG_CUSTOM_NODES
 /* Example for how to create a custom AMBA node tree. This was tested on a
  * GR712 Evaluation Board at 80MHz.
