@@ -76,14 +76,31 @@ enum proptype {
 enum pvaltype {
 	PVALTYPE_INT = 0,	/* belongs to PROPTYPE_INT */
 	PVALTYPE_COREHANDLE,	/* belongs to PROPTYPE_INT */
+	PVALTYPE_NODEHANDLE,	/* belongs to PROPTYPE_INT */
 	PVALTYPE_STRING,	/* belongs to PROPTYPE_STRING */
 };
 
+/* Node handle target */
+enum nhtarget {
+	NHTARGET_REF,	/* The nodehandle target is a named nodelabel */
+	NHTARGET_CORE	/* The nodehandle target is the node of the core */
+};
+
+/* Node label type */
+enum nltype {
+	NLTYPE_UNKNOWN	= 0,      /* Unknown */
+	NLTYPE_NODE	= 1 << 0, /* Node label */
+	NLTYPE_CORE	= 1 << 1, /* Core label */
+};
+
+#define NLTYPE_ALL (NLTYPE_NODE | NLTYPE_CORE)
+
 enum proptype pvaltype_to_proptype(enum pvaltype type)
 {
 	switch(type) {
 	case PVALTYPE_INT:
 	case PVALTYPE_COREHANDLE:
+	case PVALTYPE_NODEHANDLE:
 		return PROPTYPE_INT;
 	case PVALTYPE_STRING:
 	default:
@@ -105,8 +122,8 @@ struct propval {
 	void *intnext;  /* slist next in priv->intlist */
 	int intindex;
 
-	struct match *target_match;
-	void *chnext;  /* slist next in priv->corehandles and match->chfixups */
+	enum nhtarget nhtarget;
+	void *nhnext;  /* slist next in priv->nodehandles and match->nhfixups */
 };
 
 struct property {
@@ -133,12 +150,15 @@ struct node {
 	int id; /* Running id for nodes */
 	char *name;
 
+	char *nodelabel;
+	enum nltype nltype;
 
 	void *next; /* slist next under owning match/node */
 
 	/* Code generation */
 	void *gnext; /* slist next in global list of nodes */
 	int gindex;
+	struct slist nhfixups;
 };
 
 /* Match types are mutually exclusive */
@@ -160,12 +180,7 @@ struct match {
 	/* Bus match */
 	char *bus;
 
-	char *corelabel;
-
-
 	/* Code generation */
-	struct slist chfixups;
-
 	void *next; /* slist next pointer */
 };
 
@@ -200,11 +215,11 @@ struct scanpriv {
 	int textlen;
 
 	struct slist intlist;
-	struct slist corehandles;
+	struct slist nodehandles;
 	int intcount;
 
 	/* Code generation */
-	int chcount;
+	int nhcount;
 
 	/* Debug flags */
 	int dbg_parsing;
@@ -218,7 +233,7 @@ static void initpriv(struct scanpriv *priv) {
 	slist_init(&priv->stack, offsetof(struct stackframe, parent_frame));
 	slist_init(&priv->matches, offsetof(struct match, next));
 	slist_init(&priv->intlist, offsetof(struct propval, intnext));
-	slist_init(&priv->corehandles, offsetof(struct propval, chnext));
+	slist_init(&priv->nodehandles, offsetof(struct propval, nhnext));
 }
 
 
@@ -247,6 +262,8 @@ enum tag {
 	TAG_COREHANDLE,
 	TAG_STRING,
 	TAG_MATCHROOT,
+	TAG_NODELABEL,
+	TAG_NODEHANDLE,
 	NUMBER_OF_TAGS		/* MUST be last one */
 };
 
@@ -444,6 +461,21 @@ static char *dup_string_type(const char *str, unsigned types)
 	return dup_string(str);
 }
 
+enum tag pvaltype_to_tag(enum pvaltype pvaltype)
+{
+	switch (pvaltype) {
+	case PVALTYPE_STRING:
+		return TAG_STRING;
+	case PVALTYPE_INT:
+		return TAG_INT;
+	case PVALTYPE_COREHANDLE:
+		return TAG_COREHANDLE;
+	case PVALTYPE_NODEHANDLE:
+		return TAG_NODEHANDLE;
+	}
+	return NUMBER_OF_TAGS;
+}
+
 /* ============================================================ */
 /* Parse tree building */
 
@@ -530,18 +562,19 @@ static struct propval *new_intval(struct scanpriv *priv, struct property *p,
 }
 
 /* ref is are allowed to be NULL */
-static struct propval *new_corehandleval(struct scanpriv *priv,
-					 struct property *p, const char* ref)
+static struct propval *new_handleval(struct scanpriv *priv,
+					 struct stackframe *frame,
+					 struct property *p, const char* ref,
+					 enum nhtarget target,
+					 enum pvaltype pvaltype)
 {
 	struct propval *pv;
-
-	pv = new_propval(priv, p, PVALTYPE_COREHANDLE, ref);
+	pv = new_propval(priv, p, pvaltype, ref);
 	if (!pv)
 		return NULL;
 
-	if (!ref) /* Self referencing */
-		pv->target_match = priv->currmatch;
-	slist_add_tail(&priv->corehandles, pv);
+	pv->nhtarget = target;
+	slist_add_tail(&priv->nodehandles, pv);
 
 	return pv;
 }
@@ -568,6 +601,7 @@ static struct node *new_node(struct scanpriv *priv, struct node *parent,
 	/* Step 2 - Initialize */
 	slist_init(&n->nodes, offsetof(struct node, next));
 	slist_init(&n->properties, offsetof(struct property, next));
+	slist_init(&n->nhfixups, offsetof(struct propval, nhnext));
 	n->action = action;
 
 	if (name) {
@@ -619,9 +653,6 @@ static struct match *new_core_match(struct scanpriv *priv, enum action action)
 	struct match *m;
 
 	m = new_match(priv, MATCHTYPE_CORE, action);
-	if (m) {
-		slist_init(&m->chfixups, offsetof(struct propval, chnext));
-	}
 
 	return m;
 }
@@ -731,7 +762,7 @@ static void doc_corelabel(struct taginfo *ti)
 	printf("   A mandatory \"name\" attribute sets a label for the parent core that is used\n");
 	printf("   by <%s> tags added elsewhere to be able to reference that parent\n", ch->name);
 	printf("   core. Only one corelabel can be set per match and the label name needs to be\n");
-	printf("   unique among all corelabels.\n");
+	printf("   unique among all corelabels and nodelabels.\n");
 }
 
 static void doc_addnode(struct taginfo *ti)
@@ -749,11 +780,12 @@ static void doc_addprop(struct taginfo *ti)
 	struct taginfo *tint = &taginfos[TAG_INT];
 	struct taginfo *tch = &taginfos[TAG_COREHANDLE];
 	struct taginfo *tstr = &taginfos[TAG_STRING];
+	struct taginfo *tnh = &taginfos[TAG_NODEHANDLE];
 
 	printf("   Adds a property. A mandatory \"%s\" attribute names the property. A property\n", ATTR_NAME);
 	printf("   can be of three different kinds: empty, integer or string. A property\n");
-	printf("   becomes empty if it has no child tags. Child tags of type <%s> and\n", tint->name);
-	printf("   <%s> makes the property integer type, whereas child tags of type\n", tch->name);
+	printf("   becomes empty if it has no child tags. Child tags of type <%s>, <%s>\n", tint->name, tch->name);
+	printf("   or <%s> makes the property integer type, whereas child tags of type\n", tnh->name);
 	printf("   <%s> makes it string type. Setting more than one child tag makes the\n", tstr->name);
 	printf("   property an array of values, but it can only have integer type or string\n");
 	printf("   type values, not both.\n");
@@ -803,6 +835,27 @@ static void doc_matchroot(struct taginfo *ti)
 	printf("   Matches the root node.\n");
 }
 
+static void doc_nodelabel(struct taginfo *ti)
+{
+	struct taginfo *ch = &taginfos[TAG_NODEHANDLE];
+	const char *nh_name = ch->name;
+
+	printf("   A mandatory \"name\" attribute sets a label for the parent node that is used\n");
+	printf("   by <%s> tags added elsewhere to be able to reference that parent\n", nh_name);
+	printf("   node. Only one nodelabel can be set per node and the label name needs to be\n");
+	printf("   unique among all nodelabels and corelabels.\n");
+}
+
+static void doc_nodehandle(struct taginfo *ti)
+{
+	struct taginfo *cl = &taginfos[TAG_NODELABEL];
+
+	printf("   An integer value of an added property that contains the address of the prom\n");
+	printf("   tree node of the referenced node. To reference a node that is labeled using\n");
+	printf("   the <%s> tag, the \"%s\" attribute should have the value of that\n", cl->name, ATTR_REF);
+	printf("   label.\n");
+}
+
 static void doc_print()
 {
 	struct taginfo *ti;
@@ -1059,14 +1112,17 @@ static int start_corehandle(struct scanpriv *priv, struct stackframe *frame,
 	struct property *p;
 	struct propval *pv;
 	const char *ref;
+	enum nhtarget nhtarget = NHTARGET_REF;
 
 	p = get_property_from_stack(priv, frame);
 	if (!p)
 		return 1;
 
 	ref = get_attribute(ATTR_REF, attr); /* Allowed to be NULL */
+	if (!ref)
+		nhtarget = NHTARGET_CORE;
 
-	pv = new_corehandleval(priv, p, ref);
+	pv = new_handleval(priv, frame, p, ref, nhtarget, PVALTYPE_COREHANDLE);
 	if (!pv)
 		return 1;
 
@@ -1091,33 +1147,141 @@ static int end_string(struct scanpriv *priv, struct stackframe *frame)
 	return 0;
 }
 
+static struct node* find_node(struct node *parent,
+			      const char *label,
+			      unsigned int nltype_mask)
+{
+	struct node *child;
+	struct node *n;
+
+	if (!label || !parent || nltype_mask == NLTYPE_UNKNOWN)
+		return NULL;
+
+	/* Check if the current node match the label */
+	if (parent->nodelabel && (parent->nltype & nltype_mask) &&
+	    strcmp(parent->nodelabel, label) == 0)
+		return parent;
+
+	/* And the same for each child */
+	SLIST_FOR_EACH (&parent->nodes, child, struct node) {
+		n = find_node(child, label, nltype_mask);
+		if (n)
+			return n;
+	}
+
+	return NULL;
+}
+
+static struct node *find_any_node_by_name(struct scanpriv *priv,
+					  const char *name)
+{
+	struct match *o;
+	struct node *n;
+
+	SLIST_FOR_EACH (&priv->matches, o, struct match) {
+		/* Check that no other node use the same label */
+		n = find_node(o->placeholder, name, NLTYPE_ALL);
+		if (n)
+			return n;
+	}
+
+	return NULL;
+}
+
+static int start_label(struct scanpriv *priv, struct stackframe *frame,
+		       const char **attr, enum nltype nltype)
+{
+	struct node *node = NULL;
+	const char *name = get_required_attribute(ATTR_NAME, attr);
+
+	if (!name)
+		return 1;
+
+	/* Enforce uniqueness of labels regardless of node label type */
+	node = find_any_node_by_name(priv, name);
+	if (node) {
+		struct taginfo *ti = NULL;
+
+		switch (node->nltype) {
+		case NLTYPE_NODE:
+			ti = &taginfos[TAG_NODEHANDLE];
+			break;
+		case NLTYPE_CORE:
+			ti = &taginfos[TAG_COREHANDLE];
+			break;
+		case NLTYPE_UNKNOWN:
+			break;
+		}
+		fprintf(stderr, "A %s with name=\"%s\" is already defined\n",
+			ti ? ti->name : "<unknown tag>", name);
+		return 1;
+	}
+
+	node = get_upper_data(priv, frame);
+	if (!node) {
+		fprintf(stderr, "%s is not supported by parent tag\n",
+				frame->ti->name);
+		return 1;
+	}
+
+	/* Check if the node already have a label */
+	if (node->nodelabel) {
+		enum tag upper_tag = get_upper_tag(priv, frame);
+		struct taginfo *ti = &taginfos[upper_tag];
+		struct taginfo *nlti = &taginfos[TAG_NODELABEL];
+		struct taginfo *clti = &taginfos[TAG_CORELABEL];
+
+		if (node->nltype == nltype)
+			fprintf(stderr, "Only one <%s> is allowed under <%s>\n",
+				frame->ti->name, ti->name);
+		else
+			fprintf(stderr, "Only one of <%s> or <%s> can be defined under <%s>.\n",
+				clti->name, nlti->name, ti->name);
+
+		return 1;
+	}
+
+	node->nodelabel = dup_string(name);
+	node->nltype = nltype;
+
+	if (!node->nodelabel)
+		return 1;
+
+	return 0;
+}
+
+static int start_nodelabel(struct scanpriv *priv, struct stackframe *frame,
+			   const char **attr)
+{
+	return start_label(priv, frame, attr, NLTYPE_NODE);
+}
+
 static int start_corelabel(struct scanpriv *priv, struct stackframe *frame,
 			   const char **attr)
 {
-	struct match *m;
-	struct match *o;
-	const char *label;
+	return start_label(priv, frame, attr, NLTYPE_CORE);
+}
 
-	m = priv->currmatch;
-	if (m->corelabel) {
-		fprintf(stderr, "Corelabel already set for current match\n");
+static int start_nodehandle(struct scanpriv *priv, struct stackframe *frame,
+			    const char **attr)
+{
+	struct property *p;
+	struct propval *pv;
+	const char *ref;
+	enum nhtarget nhtarget = NHTARGET_REF;
+
+	p = get_property_from_stack(priv, frame);
+	if (!p)
+		return 1;
+
+	ref = get_attribute(ATTR_REF, attr); /* Not allowed to be NULL */
+	if (!ref) {
+		fprintf(stderr, "Nodehandle must have a 'ref' attribute\n");
 		return 1;
 	}
 
-	label = get_required_attribute(ATTR_NAME, attr);
-	if(!label)
-		return 1;
-
-	SLIST_FOR_EACH (&priv->matches, o, struct match) {
-		if (o->corelabel && strcmp(o->corelabel, label) == 0) {
-			fprintf(stderr, "Corelabel \"%s\" already defined\n",
-				label);
-			return 1;
-		}
-	}
-
-	m->corelabel = dup_string(label);
-	if (!m->corelabel)
+	pv = new_handleval(priv, frame, p, ref, nhtarget, PVALTYPE_NODEHANDLE);
+	if (!pv)
 		return 1;
 
 	return 0;
@@ -1194,6 +1358,7 @@ static int init_tags(void) {
 	valid_child(ti, TAG_ADDPROP);
 	valid_child(ti, TAG_DELPROP);
 	valid_child(ti, TAG_CORELABEL);
+	valid_child(ti, TAG_NODELABEL);
 
 	ti = init_tag(TAG_DELCORE, "del-core", start_match, NULL);
 	set_doc_function(ti, doc_delcore);
@@ -1213,12 +1378,14 @@ static int init_tags(void) {
 	set_doc_function(ti, doc_addnode);
 	valid_child(ti, TAG_ADDPROP);
 	valid_child(ti, TAG_ADDNODE);
+	valid_child(ti, TAG_NODELABEL);
 
 	ti = init_tag(TAG_ADDPROP, "add-prop", start_prop, NULL);
 	set_doc_function(ti, doc_addprop);
 	valid_child(ti, TAG_INT);
 	valid_child(ti, TAG_COREHANDLE);
 	valid_child(ti, TAG_STRING);
+	valid_child(ti, TAG_NODEHANDLE);
 
 	/*
 	 * del-prop uses the same tag parse functions as add-prop, but has no
@@ -1237,6 +1404,12 @@ static int init_tags(void) {
 	ti = init_tag(TAG_STRING, "string", trigger_text_recording, end_string);
 	set_doc_function(ti, doc_string);
 
+	ti = init_tag(TAG_NODELABEL, "nodelabel", start_nodelabel, NULL);
+	set_doc_function(ti, doc_nodelabel);
+
+	ti = init_tag(TAG_NODEHANDLE, "nodehandle", start_nodehandle, NULL);
+	set_doc_function(ti, doc_nodehandle);
+
 	for (i = 0; i < NUMBER_OF_TAGS; i++) {
 		ti = &taginfos[i];
 		ti->tag = i;
@@ -1487,17 +1660,35 @@ static int prepare_int(struct scanpriv *priv, struct propval *pv, int indent)
 	return 0;
 }
 
-static int prepare_corehandle(struct scanpriv *priv, struct propval *pv,
+static int prepare_nodehandle(struct scanpriv *priv, struct propval *pv,
 			      int indent)
 {
+	struct node *n = NULL;
 	struct match *m;
+	struct taginfo *ti = NULL;
+	enum tag tag = pvaltype_to_tag(pv->type);
+	enum nltype nltype = NLTYPE_UNKNOWN;
+
+	if (tag == NUMBER_OF_TAGS)
+		return 1;
+
+	ti = &taginfos[tag];
 
 	if (priv->dbg_parse_tree) {
 		prindent(indent);
-		if (pv->value)
-			printf("corehandle ref=\"%s\"\n", pv->value);
-		else
-			printf("corehandle to self\n");
+		switch (pv->nhtarget) {
+		case NHTARGET_REF:
+			if (!pv->value) {
+				printf("NHTARGET_REF but no ref is set!?\n");
+				return 1;
+			}
+			printf("%s ref=\"%s\"\n", ti->name, pv->value);
+			break;
+		case NHTARGET_CORE:
+			printf("%s to Node %d\n", ti->name,
+			       priv->currmatch->placeholder->id);
+			break;
+		}
 	}
 
 	/* Book a place for a dummy value to be filled in later */
@@ -1505,17 +1696,49 @@ static int prepare_corehandle(struct scanpriv *priv, struct propval *pv,
 		return 1;
 
 	/* Find match that needs to fill in value later */
-	if (!pv->value) {
-		m = priv->currmatch;
-	} else {
-		SLIST_FOR_EACH (&priv->matches, m, struct match)
-			if (m->corelabel && strcmp(pv->value, m->corelabel) == 0)
-				goto found;
-		fprintf(stderr, "Corelabel \"%s\" undefined\n", pv->value);
-		return 1;
+	switch (pv->nhtarget) {
+	case NHTARGET_CORE:
+		n = priv->currmatch->placeholder;
+		break;
+	case NHTARGET_REF:
+		if (!pv->value) {
+			fprintf(stderr, "'Missing reference in %s.\n",
+				ti->name);
+			return 1;
+		}
+
+		/* Get the node label type for the property type */
+		switch (pv->type) {
+		case PVALTYPE_NODEHANDLE:
+			nltype = NLTYPE_NODE;
+			break;
+		case PVALTYPE_COREHANDLE:
+			nltype = NLTYPE_CORE;
+			break;
+		case PVALTYPE_INT:
+		case PVALTYPE_STRING:
+			return 1;
+		}
+
+		/* Try to find a matching node with the label and type */
+		SLIST_FOR_EACH (&priv->matches, m, struct match) {
+			n = find_node(m->placeholder, pv->value, nltype);
+			if (n)
+				break;
+		}
+
+		if (!n) {
+			fprintf(stderr, "%s \"%s\" undefined\n", ti->name,
+				pv->value);
+			return 1;
+		}
+		break;
 	}
-found:
-	slist_add_tail(&m->chfixups, pv);
+
+	if (!n)
+		return 1;
+
+	slist_add_tail(&n->nhfixups, pv);
 
 	return 0;
 }
@@ -1528,11 +1751,14 @@ static int prepare_propval(struct scanpriv *priv, struct propval *pv,
 		return prepare_int(priv, pv, indent);
 
 	case PVALTYPE_COREHANDLE:
-		return prepare_corehandle(priv, pv, indent);
+	case PVALTYPE_NODEHANDLE:
+		return prepare_nodehandle(priv, pv, indent);
 
 	case PVALTYPE_STRING:
-		if (priv->dbg_parse_tree)
+		if (priv->dbg_parse_tree) {
+			prindent(indent);
 			printf("string: \"%s\"\n", pv->value);
+		}
 		return 0;
 	}
 
@@ -1604,11 +1830,26 @@ static int prepare_match(struct scanpriv *priv, struct match *m)
 
 	if (priv->dbg_parse_tree) {
 		printf("Match %d: ", m->id);
-		if (m->vendor)
-			printf("vendor=%s device=%s index=%s\n",
-			       m->vendor, m->device, m->index);
-		else if (m->coreindex)
-			printf("coreindex=%s\n", m->coreindex);
+		switch (m->matchtype) {
+		case MATCHTYPE_CORE:
+			if (m->vendor)
+				printf("vendor=%s device=%s index=%s\n",
+				       m->vendor, m->device, m->index);
+			else if (m->coreindex)
+				printf("coreindex=%s\n", m->coreindex);
+			break;
+		case MATCHTYPE_BUS:
+			if (m->bus)
+				printf("Bus %s", m->bus);
+			printf("\n");
+			break;
+		case MATCHTYPE_ROOT:
+			printf("Root\n");
+			break;
+		case MATCHTYPE_NONE:
+			printf("\n");
+			break;
+		}
 	}
 
 	return prepare_node(priv, m->placeholder, 1);
@@ -1644,7 +1885,6 @@ static int generate_matchline(FILE *fout, struct scanpriv *priv,
 {
 	char *flags1;
 	char *flags2;
-	int chfixcount;
 
 	FPRINTF(fout, "/* Match %d */\t{", m->id);
 	flags1 = NULL;
@@ -1671,17 +1911,7 @@ static int generate_matchline(FILE *fout, struct scanpriv *priv,
 	}
 
 	FPRINTF(fout, "%s%s, ", flags1, flags2);
-	FPRINTF(fout, "%d, ", m->placeholder->id);
-
-	/*
-	 * Index into list of corehandles that points to the core of the
-	 * match. The list of corehandles contains indices to values needs to be
-	 * filled in with addresses of the prom node for the core
-	 */
-	chfixcount = SLIST_SIZE(&m->chfixups);
-	FPRINTF(fout, "%d, ", priv->chcount); /* First chfixup */
-	FPRINTF(fout, "%d},\n", chfixcount); /* Number of chfixups */
-	priv->chcount += chfixcount;
+	FPRINTF(fout, "%d},\n", m->placeholder->id);
 
 	return 0;
 }
@@ -1694,8 +1924,6 @@ static int generate_matches(FILE *fout, struct scanpriv *priv)
 	FPRINTF(fout, " * Each entry consists of:\n");
 	FPRINTF(fout, " *   vendor, device, index, coreindex, bus, flags,\n");
 	FPRINTF(fout, " *   index of placeholder node in xml_nodes[], \n");
-	FPRINTF(fout, " *   first corehandle to fix up,\n");
-	FPRINTF(fout, " *   # of corehandles to fix up\n");
 	FPRINTF(fout, " */\n");
 
 	FPRINTF(fout, "struct xml_match xml_matches[] = {\n");
@@ -1707,25 +1935,39 @@ static int generate_matches(FILE *fout, struct scanpriv *priv)
 	return 0;
 }
 
-static int generate_chfixups(FILE *fout, struct scanpriv *priv)
+static int generate_nhfixups_for_node(struct node *parent, FILE *fout)
+{
+	static int first = 1;
+	struct node *child;
+	struct propval *pv;
+
+	if (!parent)
+		return 0;
+
+	SLIST_FOR_EACH (&parent->nhfixups, pv, struct propval) {
+		if (first)
+			first = 0;
+		else
+			FPRINTF(fout, ",");
+
+		FPRINTF(fout, "%d", pv->intindex);
+	}
+
+	SLIST_FOR_EACH (&parent->nodes, child, struct node)
+		generate_nhfixups_for_node(child, fout);
+
+	return 0;
+}
+
+static int generate_nhfixups(FILE *fout, struct scanpriv *priv)
 {
 	struct match *m;
-	struct propval *pv;
-	int first = 1;
 
-	FPRINTF(fout, "/* Indices for corehandles to be fixed up in xml_values[] */\n");
-	FPRINTF(fout, "int xml_corehandles[] = {");
+	FPRINTF(fout, "/* Indices for nodehandles to be fixed up in xml_values[] */\n");
+	FPRINTF(fout, "int xml_nodehandles[] = {");
 
-	SLIST_FOR_EACH (&priv->matches, m, struct match) {
-		SLIST_FOR_EACH (&m->chfixups, pv, struct propval) {
-			if (first) {
-				first = 0;
-			} else {
-				FPRINTF(fout, ",");
-			}
-			FPRINTF(fout, "%d", pv->intindex);
-		}
-	}
+	SLIST_FOR_EACH (&priv->matches, m, struct match)
+		generate_nhfixups_for_node(m->placeholder, fout);
 
 	FPRINTF(fout, "};\n\n");
 
@@ -1737,7 +1979,7 @@ static int generate_intlist(FILE *fout, struct scanpriv *priv)
 	struct propval *pv;
 	int first = 1;
 
-	FPRINTF(fout, "/* Ingeger property values */\n");
+	FPRINTF(fout, "/* Integer property values */\n");
 	FPRINTF(fout, "int xml_values[] = {");
 
 	SLIST_FOR_EACH (&priv->intlist, pv, struct propval) {
@@ -1745,7 +1987,8 @@ static int generate_intlist(FILE *fout, struct scanpriv *priv)
 
 		if (pv->type == PVALTYPE_INT)
 			val = pv->value;
-		else if (pv->type == PVALTYPE_COREHANDLE)
+		else if (pv->type == PVALTYPE_COREHANDLE ||
+				pv->type == PVALTYPE_NODEHANDLE)
 			val = CHFIXUP_DUMMY;
 		if (!val)
 			continue;
@@ -1814,7 +2057,7 @@ static int generate_propline(FILE *fout, struct scanpriv *priv,
 		FPRINTF(fout, "PROPA_RM%s(\"%s\"),\n", endstr, p->name);
 	} else { /* ACTION_ADD */
 		FPRINTF(fout, "PROPA_PTR%s(\"%s\", ", endstr, p->name);
-		if (p->type == PROPTYPE_INT) { /* Includes corehandles */
+		if (p->type == PROPTYPE_INT) { /* Includes {core|node}handles */
 			generate_intpropval(fout, priv, p);
 		} else if (p->type == PROPTYPE_STRING) {
 			generate_stringpropval(fout, priv, p);
@@ -1876,6 +2119,7 @@ static int generate_nodelines(FILE *fout, struct scanpriv *priv,
 	struct node *firstchild;
 	struct node *child;
 	struct property *firstprop;
+	int nhfixcount;
 
 	firstchild = SLIST_HEAD(&n->nodes, struct node);
 	firstprop = SLIST_HEAD(&n->properties, struct property);
@@ -1898,11 +2142,24 @@ static int generate_nodelines(FILE *fout, struct scanpriv *priv,
         else
 		FPRINTF(fout, "NULL,           ");
 
-	FPRINTF(fout,"%d),", SLIST_SIZE(&n->properties));
-	if (n->name)
+	FPRINTF(fout,"%d, ", SLIST_SIZE(&n->properties));
+	/*
+	 * Index into list of corehandles that points to the core of the
+	 * match. The list of corehandles contains indices to values needs to be
+	 * filled in with addresses of the prom node for the core
+	 */
+	nhfixcount = SLIST_SIZE(&n->nhfixups);
+	FPRINTF(fout, "%d, ", priv->nhcount); /* First nhfixup */
+	FPRINTF(fout, "%d, ", nhfixcount); /* Number of nhfixups */
+	priv->nhcount += nhfixcount;
+
+	if (n->name) {
+		FPRINTF(fout, "0),"); /* flags */
 		FPRINTF(fout," /* \"%s\" */", n->name);
-	else
+	} else {
+		FPRINTF(fout, "XMLNODE_PLH),"); /* flags */
 		FPRINTF(fout," /* placeholder node */");
+	}
 	FPRINTF(fout,"\n");
 
 	SLIST_FOR_EACH (&n->nodes, child, struct node) {
@@ -1917,12 +2174,12 @@ static int generate_nodelist(FILE *fout, struct scanpriv *priv)
 {
 	struct match *m;
 
-	FPRINTF(fout, "#define XMLNODE_INIT(c, s, p, pcnt) \\\n");
-	FPRINTF(fout, "\t{{(struct node *)c,(struct node *)s,(struct prop *)p}, pcnt}\n\n");
+	FPRINTF(fout, "#define XMLNODE_INIT(c, s, p, pcnt, nhstart, nhcnt, flags) \\\n");
+	FPRINTF(fout, "\t{{(struct node *)c,(struct node *)s,(struct prop *)p}, pcnt, nhstart, nhcnt, flags}\n\n");
 
 	FPRINTF(fout, "/*\n");
 	FPRINTF(fout, " * Each entry consists of:\n");
-	FPRINTF(fout, " *   first child, next sibling, first property, # of properties\n");
+	FPRINTF(fout, " *   first child, next sibling, first property, # of propertien, start of nodehandle, # of nodehandles, flags\n");
 	FPRINTF(fout, " */\n");
 
 	FPRINTF(fout, "struct xml_node xml_nodes[] = {\n");
@@ -1947,7 +2204,7 @@ static int generate_code(FILE *fout, struct scanpriv *priv)
 
 	if (generate_matches(fout, priv))
 		return 1;
-	if (generate_chfixups(fout, priv))
+	if (generate_nhfixups(fout, priv))
 		return 1;
 	if (generate_intlist(fout, priv))
 		return 1;
@@ -1958,6 +2215,8 @@ static int generate_code(FILE *fout, struct scanpriv *priv)
 
 	FPRINTF(fout, "#define XML_MATCHCOUNT %d\n", priv->matchcount);
 	FPRINTF(fout, "int xml_matchcount = XML_MATCHCOUNT;\n");
+	FPRINTF(fout, "#define XML_NODECOUNT %d\n", priv->nodecount);
+	FPRINTF(fout, "int xml_nodecount = XML_NODECOUNT;\n");
 
 	return 0;
 }
