@@ -3595,7 +3595,38 @@ extern struct xml_match xml_matches[];
 extern struct xml_node xml_nodes[];
 extern int xml_values[];
 extern int xml_corehandles[];
+extern int xml_nodehandles[];
 extern int xml_matchcount;
+extern int xml_nodecount;
+
+
+void xml_nodehandle_apply(struct xml_node *xn, struct node *node)
+{
+	int i;
+	/* Fill in nodehandle values for which this node is the
+	 * target
+	 */
+	for (i = 0; i < xn->nhtarget_count; i++) {
+		int vindex = xml_nodehandles[xn->nhtarget_start + i];
+		xml_values[vindex] = (int)node;
+	}
+}
+
+void xml_nodehandle_init(void)
+{
+	int i;
+	struct xml_node *xn;
+
+	for (i = 0; i < xml_nodecount; i++) {
+		xn = &xml_nodes[i];
+		/* Skip placeholder nodes as we don't know the address */
+		if (xn->flags & XMLNODE_PLH)
+			continue;
+
+		/* Set value to which this node is target */
+		xml_nodehandle_apply(xn, &xn->node);
+	}
+}
 
 void xml_nodes_init_hook(struct ambapp_bus *ainfo)
 {
@@ -3623,6 +3654,7 @@ void xml_nodes_init_hook(struct ambapp_bus *ainfo)
 		}
 		core = core->next;
 	}
+	xml_nodehandle_init();
 }
 
 /*
@@ -3668,6 +3700,7 @@ void xml_xmlnode_apply(struct node *node, struct xml_node *xn,
 	 */
 	node_add_child(node, xn->node.child);
 
+	xml_nodehandle_apply(xn, node);
 }
 
 int xml_core_matches(ambapp_core *core, struct xml_match *m)
@@ -3702,7 +3735,7 @@ int xml_core_matches(ambapp_core *core, struct xml_match *m)
 int xml_node_create_hook(ambapp_core *core, struct node *node,
 			 struct prop_std **lastprop)
 {
-	int i, mi;
+	int mi;
 	struct xml_match *m;
 	struct xml_node *xn;
 
@@ -3717,16 +3750,6 @@ int xml_node_create_hook(ambapp_core *core, struct node *node,
 
 		xn = &xml_nodes[m->placeholder];
 		xml_xmlnode_apply(node, xn, 1, lastprop);
-
-		/*
-		 * Fill in corehandle values for which this core is the
-		 * target
-		 */
-		for (i = 0; i < m->chtarget_count; i++) {
-			int vindex = xml_corehandles[m->chtarget_start + i];
-
-			xml_values[vindex] = (int)node;
-		}
 	}
 
 	return 0;
