@@ -24,7 +24,7 @@
  * Any changes/bug-fixes should be made in the script, then rerun the script
  * to create a new file.
  *
- * Created from GRLIB revision 4297
+ * Created from GRLIB revision 4299
  */
 
 #include <ambapp_names.h>
@@ -264,6 +264,10 @@ static struct ambapp_device_name GAISLER_devices[] =
   {GAISLER_GRLSEDC, "GRLSEDC", "Lattice CRAM Scrubber"},
   {GAISLER_MDIO_CTRL, "MDIO_CTRL", "Ethernet MDIO Controller"},
   {GAISLER_OTP_WRPR, "OTP_WRPR", "OTP control wrapper"},
+  {GAISLER_LATTICE_DDR3, "LATTICE_DDR3", "Lattice DDR3 memory controller"},
+  {GAISLER_GR1553B_LRAM, "GR1553B_LRAM", "1553B Interface with local SRAM"},
+  {GAISLER_GRETH2, "GRETH2", "Ethernet Gbit MAC 2"},
+  {GAISLER_GR765THS, "GR765THS", "GR765 Temp/Voltage sensor"},
   {0, NULL, NULL}
 };
 
@@ -469,6 +473,7 @@ static struct ambapp_device_name ORBITA_devices[] =
 /* Vendor SYNOPSYS devices */
 static struct ambapp_device_name SYNOPSYS_devices[] =
 {
+  {SYNOPSYS_PCIE_CTRL, "PCIE_CTRL", "PCI Express controller"},
   {0, NULL, NULL}
 };
 
@@ -543,6 +548,7 @@ static struct ambapp_device_name ST_devices[] =
   {GAISLER_AHBMST_EM, "AHBMST_EM", "AMBA Master Emulator"},
   {GAISLER_TEST_1X2, "TEST_1X2", "HAPS TEST_1x2 interface"},
   {ST_MIPHY, "MIPHY", "MIPHY transceiver"},
+  {ST_AIB, "AIB", "AIB (die2die)"},
   {0, NULL, NULL}
 };
 
@@ -646,6 +652,17 @@ static struct ambapp_device_name ACTEL_devices[] =
 };
 
 
+/* Vendor BG devices */
+static struct ambapp_device_name BG_devices[] =
+{
+  {BG_RIXCORE, "RIXCORE", "RIXCore SF GNSS Processor"},
+  {BG_DIAGIF, "DIAGIF", "Diagnosis I/F"},
+  {BG_WDOG, "WDOG", ""},
+  {BG_SPI, "SPI", ""},
+  {0, NULL, NULL}
+};
+
+
 /* Vendor APPLECORE devices */
 static struct ambapp_device_name APPLECORE_devices[] =
 {
@@ -706,6 +723,28 @@ static struct ambapp_device_name EMBEDDIT_devices[] =
 };
 
 
+/* Vendor BRAINCHIP devices */
+static struct ambapp_device_name BRAINCHIP_devices[] =
+{
+  {0, NULL, NULL}
+};
+
+
+/* Vendor MAGICS devices */
+static struct ambapp_device_name MAGICS_devices[] =
+{
+  {0, NULL, NULL}
+};
+
+
+/* Vendor LEAT devices */
+static struct ambapp_device_name LEAT_devices[] =
+{
+  {LEAT_SPLEAT, "SPLEAT", ""},
+  {0, NULL, NULL}
+};
+
+
 /* Vendor NASA_GSFC devices */
 static struct ambapp_device_name NASA_GSFC_devices[] =
 {
@@ -762,6 +801,7 @@ static struct ambapp_ids vendors[] =
   {VENDOR_AAC, "AAC", "AAC Microtec", AAC_devices},
   {VENDOR_UC_BERKELEY, "UC_BERKELEY", "UC, Berkeley", UC_BERKELEY_devices},
   {VENDOR_ACTEL, "ACTEL", "Microsemi Corporation", ACTEL_devices},
+  {VENDOR_BG, "BG", "Beyond Gravity GmbH", BG_devices},
   {VENDOR_APPLECORE, "APPLECORE", "AppleCore", APPLECORE_devices},
   {VENDOR_C3E, "C3E", "TU Braunschweig C3E", C3E_devices},
   {VENDOR_CBKPAN, "CBKPAN", "CBK PAN", CBKPAN_devices},
@@ -769,6 +809,9 @@ static struct ambapp_ids vendors[] =
   {VENDOR_CETON, "CETON", "Ceton Corporation", CETON_devices},
   {VENDOR_SSTL, "SSTL", "SSTL", SSTL_devices},
   {VENDOR_EMBEDDIT, "EMBEDDIT", "Embedd.it", EMBEDDIT_devices},
+  {VENDOR_BRAINCHIP, "BRAINCHIP", "Brainchip", BRAINCHIP_devices},
+  {VENDOR_MAGICS, "MAGICS", "Magics", MAGICS_devices},
+  {VENDOR_LEAT, "LEAT", "LEAT", LEAT_devices},
   {VENDOR_NASA_GSFC, "NASA_GSFC", "NASA GSFC", NASA_GSFC_devices},
   {VENDOR_AZST, "AZST", "AZST", AZST_devices},
   {0, NULL, NULL, NULL}
